# How to compile fingerbank-collector for various architectures using the OpenWRT sources.

NOTE: Built for CentOS 7

## RPM packages needed

yum install subversion binutils bzip2 gcc gcc-c++ gawk gettext flex ncurses-devel zlib-devel zlib-static make patch unzip perl-ExtUtils-MakeMaker glibc glibc-devel glibc-static quilt ncurses-libs sed sdcc intltool sharutils bison wget git-core openssl-devel xz 

## Download openwrt

```
cd /usr/src
git clone https://github.com/openwrt/openwrt.git
cd openwrt
git checkout v18.06.2
```

## Configure openwrt

```
make menuconfig
```

ARM: 
  Target System -> Marvell EBU Armada
  Subtarget -> Marvell Armada 37x/38x/XP
  Target Profile -> Linksys WRT1900AC (Mamba)
  Libraries -> 
    * libnl
    * libnfnetlink
    * libpcap

ARM64 (ARM8): 
  Target System -> Marvell EBU Armada
  Target Profile -> Marvell Armada 3700LP (ARM64)
  Libraries -> 
    * libnl
    * libnfnetlink
    * libpcap

i386: 
  Target System -> x86
  Subtarget -> Generic
  Target Profile -> Generic
  Libraries -> 
    * libnl
    * libpcap

x86_64:
  Target System -> x86
  Subtarget -> x86_64
  Target Profile -> Generic
  Libraries -> 
    * libnl
    * libpcap

mips_24kc (Mojo C-65):
  Target System -> Atheros AR7xxx/AR9xxx
  Subtarget -> Generic
  Target Profile -> Default Profile (all drivers)
  Libraries -> 
    * libnl
    * libpcap

mipsle (GL-MT300N-V2):
  Target System -> RAMIPS
  Subtarget -> mt76x8
  Libraries ->
    * libnl
    * libpcap


## Build the bundle

NOTE: This will take a long time

`make -j $(nproc --all)`

If the above fails and you're running as root: `FORCE_UNSAFE_CONFIGURE=1 make`

## Build

  * ARM7: `make fingerbank-collector-arm7`
  * ARM5: `make fingerbank-collector-arm5`
  * MIPS64: `make fingerbank-collector-mips64`
  * i386: `make fingerbank-collector-i386`
  * ARM64: `make fingerbank-collector-arm-aarch64`

## OPTIONAL: Strip the binary (change toolchain path depending on what architecture you're compiling)

`/usr/src/openwrt/staging_dir/toolchain-arm_cortex-a9+vfpv3_gcc-4.8-linaro_uClibc-0.9.33.2_eabi/arm-openwrt-linux-uclibcgnueabi/bin/strip fingerbank-collector`

# Building with nDPI support

## Basic setup

Setup your build root according to the instructions in 'Configure openwrt'

Setup nDPI in your openwrt directory:

```
cd /usr/src/openwrt
./scripts/feeds update -a
mkdir -p ./feeds/packages/libs/libndpi/
# The file below is intentionally kept public due to the fact it's GPL licensed
curl https://gist.githubusercontent.com/julsemaan/4886bd3a886f185130d2be69c0c77820/raw/223e539d9c96b11224e41c635103ada72560eb4a/libndpi-3.2-openwrt > ./feeds/packages/libs/libndpi/Makefile
./scripts/feeds update -a
./scripts/feeds install libndpi
```

## Add libndpi and rebuild your openwrt SDK

  * make menuconfig
    * Add libndpi to the libraries
  * make

## Compile your collector

```
BUILD_TAGS="dpi dpi_godpi" make fingerbank-collector-arm7
```

## Using the collector with nDPI

You will need to install libndpi 3.2 (which has been compiled and package in the prior steps)

The ipk package is in `/usr/src/openwrt/bin/packages/mipsel_24kc/packages/libndpi_3.2-1_mipsel_24kc.ipk` (adjust path based on your architecture target)

curl https://gist.githubusercontent.com/julsemaan/4886bd3a886f185130d2be69c0c77820/raw/64a459758464e162f0b81425537775ef189e595b/libndpi-3.2-openwrt > ./feeds/packages/libs/libndpi/Makefile
./scripts/feeds update -a
make
```
