package api

import (
	fccommon "git.inverse.ca/inverse/fingerbank-collector/common"

	"github.com/gin-gonic/gin"
)

var collector *fccommon.FingerbankCollector

func Setup(fc *fccommon.FingerbankCollector) *gin.Engine {
	router := gin.Default()
	router.SetHTMLTemplate(listEndpointDataHTML)

	collector = fc

	authorized := router.Group("/", TokenAuthMiddleware())
	SetupEndpointData(authorized)
	SetupSync(authorized)
	SetupReplication(authorized)
	SetupDebug(authorized)

	return router
}

func SetupDebug(router *gin.RouterGroup) {
	router.GET("/debug", getDebug)
}

func SetupEndpointData(router *gin.RouterGroup) {
	router.GET("/endpoints_data", listEndpointData)
	router.GET("/endpoints_data/details", listEndpointDataDetails)
	router.GET("/endpoint_data", listEndpointData)
	router.GET("/endpoint_data/:deviceId", getEndpointData)
	router.GET("/endpoint_data/:deviceId/details", endpointDataDetails)
	router.GET("/endpoint_data/:deviceId/print_query", endpointDataPrintQuery)
	router.GET("/endpoint_data/:deviceId/analyse", endpointDataAnalyse)
	router.GET("/endpoint_data/:deviceId/is_a/:otherDeviceId", endpointDataIsA)
	router.PATCH("/endpoint_data/:deviceId", updateEndpointData)
	router.DELETE("/endpoint_data/:deviceId", deleteEndpointData)
}

func SetupSync(router *gin.RouterGroup) {
	clusterSync := router.Group("/sync", ClusterIdMiddleware())
	clusterSync.GET("/endpoint_data", syncEndpointData)
	clusterSync.GET("/endpoint_data/:deviceId/oda_data", syncEndpointDataOdaData)
	clusterSync.GET("/ip_maps", syncIpMaps)
	clusterSync.GET("/endpoints_cache", syncEndpointsCache)
}

func SetupReplication(router *gin.RouterGroup) {
	router.GET("/replication/peers", listPeers)
}
