package api

import (
	"net/http"

	"github.com/gin-gonic/gin"

	fccommon "git.inverse.ca/inverse/fingerbank-collector/common"
)

func syncEndpointData(c *gin.Context) {
	c.Writer.Header().Add("Content-Type", "application/json")
	c.Writer.WriteHeader(http.StatusOK)
	collector.WriteEndpointsData(c.Writer)
}

func syncIpMaps(c *gin.Context) {
	lockid := collector.EndpointsDataRLock()
	defer collector.EndpointsDataRUnlock(lockid)

	c.JSON(http.StatusOK, collector.IpMaps())
}

func syncEndpointsCache(c *gin.Context) {
	c.JSON(http.StatusOK, collector.EndpointsCache)
}

func syncEndpointDataOdaData(c *gin.Context) {
	if ed, err := findEndpointData(c, c.Param("deviceId"), true); err != nil {
		c.JSON(http.StatusNotFound, gin.H{"error": err.Error()})
	} else {
		lockid := ed.RLock()
		defer ed.RUnlock(lockid)
		ahcMap := map[string]map[string]int{}
		if ahc, found := collector.EndpointsCache.GetEndpointAllHostsCache(ed.GetMac()); found {
			lockid := ahc.RLock()
			defer ahc.RUnlock(lockid)
			ahcMap = ahc.ToMap()
		}
		c.JSON(http.StatusOK, fccommon.EndpointDataODA{EndpointData: ed, AllHostsCache: ahcMap})
	}
}
