package boot_sequence

import (
	"testing"
)

func TestBootSequence(t *testing.T) {
	bs := NewBootSequence()
	bs.Add("TEST")
	if len(bs.Events) != 0 {
		t.Fatalf("NewBootSequence added a start")
	}

	bs.Start()
	if len(bs.Events) != 1 {
		t.Errorf("Start added")
	}

	bs.Add("TEST")
	if len(bs.Events) != 2 {
		t.Errorf("Start added")
	}

	bs.Add("TEST")
	if len(bs.Events) != 3 {
		t.Errorf("NewBootSequence added")
	}

}

type parseEventTest struct {
	name         string
	event        string
	expectedTS   int64
	expectedData string
}

func (test *parseEventTest) Test(t *testing.T) {
	ts, data := ParseEvent(test.event)
	if ts != test.expectedTS {
		t.Errorf("ts %d does not match expect value %d", ts, test.expectedTS)
	}
	if data != test.expectedData {
		t.Errorf("data '%s' does not match expect value '%s'", data, test.expectedData)
	}
}

func TestParseEvent(t *testing.T) {
	for _, test := range []parseEventTest{
		{
			name:         "Empty Event",
			event:        "",
			expectedTS:   0,
			expectedData: "",
		},
		{
			name:         "No Timestamp",
			event:        "sss",
			expectedTS:   0,
			expectedData: "sss",
		},
		{
			name:         "With Timestamp",
			event:        "1:sss",
			expectedTS:   1,
			expectedData: "sss",
		},
		{
			name:         "Bad Timestamp",
			event:        "aa:sss",
			expectedTS:   0,
			expectedData: "aa:sss",
		},
	} {
		test := test
		t.Run(test.name, test.Test)
	}
}
