package fccommon

import (
	"encoding/json"
	"fmt"
	"strconv"
	"testing"

	"github.com/inverse-inc/go-utils/sharedutils"
)

func TestAllHostsCache(t *testing.T) {
	ahc := NewAllHostsCache(5)

	expected := []string{}
	for i := 0; i < 5; i++ {
		stri := strconv.Itoa(i)
		k := "1.2.3." + stri
		ahc.Add(k, fmt.Sprintf("tcp:%d", i))
		expected = append(expected, k)
	}

	ihm := ahc.ToMap()

	if len(ihm) != len(expected) {
		t.Error("Length of ip host map is greater than the max size")
	}

	for _, k := range expected {
		if _, found := ihm[k]; !found {
			t.Error("Unable to find key in ip host map", k)
		}
	}

	legacyihm := ahc.ToLegacyMap()

	for _, k := range expected {
		if _, found := legacyihm[k]; !found {
			t.Error("Unable to find key in ip host map", k)
		}
	}

}

func TestAllHostsCacheJSON(t *testing.T) {
	ahc := NewAllHostsCache(2)
	values := map[string]string{
		"1.2.3.4": "1234",
		"2.3.4.5": "tcp:2345",
	}
	for k, v := range values {
		ahc.Add(k, v)
	}
	d, err := json.Marshal(ahc)
	sharedutils.CheckError(err)

	ahc2 := &AllHostsCache{}
	err = json.Unmarshal(d, ahc2)

	if ahc.cache.MaxSize() != ahc2.cache.MaxSize() {
		t.Error("maxSize wasn't restored properly")
	}

	for k, v := range values {
		if foundV, found := ahc2.Get(k); !found || 1 != foundV[v] {
			t.Error("Key", k, "wasn't found or value was unexpected. Expected", v, "Found", foundV)
		}
	}

	sharedutils.CheckError(err)
}
