//go:build dpi_qosmos
// +build dpi_qosmos

package fccommon

import (
	"flag"
	"fmt"

	"git.inverse.ca/inverse/fingerbank-collector/dpi"
	"github.com/inverse-inc/go-utils/sharedutils"
)

var dpiIgnoreSignatures = map[string]bool{
	dpi.QosmosPrefix + "base":    true,
	dpi.QosmosPrefix + "ip":      true,
	dpi.QosmosPrefix + "llc":     true,
	dpi.QosmosPrefix + "unknown": true,
}

var qosmosWorkers = flag.Int(
	"qosmos-workers",
	sharedutils.EnvOrDefaultInt("COLLECTOR_QOSMOS_WORKERS", 1),
	`Amount of workers for the qosmos engine`,
)

func (fc *FingerbankCollector) initDPI() {
	var err error
	fc.dpiEngine, err = dpi.CreateQosmosEngine(*qosmosWorkers)
	if err != nil {
		fc.logger.Error(fmt.Sprintf("Unable to create DPI engine: %s", err))
		fc.dpiEngine = nil
		return
	}
	fc.dpiEngine.SetCallback(fc.processPacketDPI)
}
