package fccommon

import (
	"regexp"
	"testing"
)

func TestHostValidator(t *testing.T) {
	// Any host should pass an empty host validator
	hv := NewHostValidator(make([]string, 0), make([]*regexp.Regexp, 0))

	if !hv.Validate("vidange.com") {
		t.Error("Host was not valid even though it should have been")
	}

	// Test host validator with strings
	hv = NewHostValidator(
		[]string{
			"test.com",
			"test2.com",
		},
		make([]*regexp.Regexp, 0),
	)

	if !hv.Validate("test.com") || !hv.Validate("test2.com") {
		t.Error("Host was not valid even though it should have been")
	}

	if hv.Validate("vidange.com") {
		t.Error("Host was valid even though it shouldn't have been")
	}

	// Test host validator with regexp
	hv = NewHostValidator(
		make([]string, 0),
		[]*regexp.Regexp{
			regexp.MustCompile(`^centos\.mirror\..*`),
			regexp.MustCompile(`^test\.mirror\..*`),
		},
	)

	if !hv.Validate("centos.mirror.hello.com") || !hv.Validate("test.mirror.com") {
		t.Error("Host was not valid even though it should have been")
	}

	if hv.Validate("vidange.com") {
		t.Error("Host was valid even though it shouldn't have been")
	}

	// Test host validator with regexp and string
	hv = NewHostValidator(
		[]string{
			"test.com",
			"test2.com",
		},
		[]*regexp.Regexp{
			regexp.MustCompile(`^centos\.mirror\..*`),
			regexp.MustCompile(`^test\.mirror\..*`),
		},
	)

	if !hv.Validate("test.com") || !hv.Validate("test2.com") {
		t.Error("Host was not valid even though it should have been")
	}

	if !hv.Validate("centos.mirror.hello.com") || !hv.Validate("test.mirror.com") {
		t.Error("Host was not valid even though it should have been")
	}

	if hv.Validate("vidange.com") {
		t.Error("Host was valid even though it shouldn't have been")
	}
}
