package fcreplication

import "time"

type TrackablePeer struct {
	deadErrorThreshold uint
	deadDuration       time.Duration

	errorCount uint

	retryAt time.Time
}

func (tp *TrackablePeer) InitTrackablePeer(deadErrorThreshold uint, deadDuration time.Duration) {
	tp.deadErrorThreshold = deadErrorThreshold
	tp.deadDuration = deadDuration
}

func (tp *TrackablePeer) ShouldQuery() bool {
	// If we're bound to retry, this supersedes the error threshold
	if time.Now().After(tp.retryAt) {
		return true
	}

	// If we're under the dead threshold, then we're all good
	if tp.errorCount < tp.deadErrorThreshold {
		return true
	}

	// If we're here, nothing positive allows to use this server
	return false
}

func (tp *TrackablePeer) RecordError() {
	tp.errorCount += 1
	tp.retryAt = time.Now().Add(tp.deadDuration)
}

func (tp *TrackablePeer) RecordSuccess() {
	tp.errorCount = 0
}
