package main

import (
	"fmt"
	"reflect"
	"strings"

	fccommon "git.inverse.ca/inverse/fingerbank-collector/common"
)

func main() {
	ed := fccommon.NewEndpointData(fccommon.DummyEndpointDataUpdateHook, fccommon.DummyEndpointDataAnalysisHook, fccommon.DummyEndpointDataRunsInLiveReplicationHook)
	v := reflect.ValueOf(ed).Elem()
	t := v.Type()

	fmt.Printf(`
// This file is automatically generated using tools/generate_delta_endpoint_data.go 
// Changes to this file will be overwritten on the next build
package fccommon

import (
	"time"
	"strconv"
)

`)
	fmt.Printf("type DeltaEndpointData struct {\n")
	for i := 0; i < v.NumField(); i++ {
		rfield := v.Field(i)
		rfieldType := t.Field(i)
		fieldName := rfieldType.Name

		jsonTag := rfieldType.Tag.Get("json")
		jsonTag += ",omitempty"

		// Only manage exported fields
		if strings.ToUpper(string(fieldName[0])) != string(fieldName[0]) || fieldName == "BootSequence" || fieldName == "ODAAllHostsCache" {
			continue
		}

		fieldType := rfield.Type()
		fieldTypeStr := fieldType.String()
		if fieldTypeStr == "time.Time" {
			fieldTypeStr = "*" + fieldTypeStr
		}

		fmt.Printf("%s %s `json:\"%s\"`\n", fieldName, fieldTypeStr, jsonTag)
	}
	fmt.Printf("}\n\n")
	fmt.Printf(`
func addEventsMapIntInt(events *[]string, timestamp, field string, v1, v2 map[int]int) {
    for k, v := range v2 {
        if val, found := v1[k]; !found || v != val {
            *events = append(*events, timestamp + ":" + field + ":" + strconv.Itoa(k))
        }
    }
}

func addEventsMapStringInt(events *[]string, timestamp, field string, v1, v2 map[string]int) {
    for k, v := range v2 {
        if val, found := v1[k]; !found || v != val {
            *events = append(*events, timestamp + ":" + field + ":" + k)
        }
    }
}

func addEventsMapStringIntInt(events *[]string, timestamp, field string, v1, v2 map[string]map[int]int) {
    for k, v := range v2 {
        addEventsMapIntInt(events, timestamp, field + ":" + k, v1[k], v)
    }
}

func addEventsMapStringStringInt(events *[]string, timestamp, field string, v1, v2 map[string]map[string]int) {
    for k, v := range v2 {
        addEventsMapStringInt(events, timestamp, field + ":" + k, v1[k], v)
    }
}

func addEventsString(events *[]string, timestamp, field string, v1, v2 string) {
    if v2 != "" && v1 != v2 {
        *events = append(*events, string(timestamp + ":" + field + ":" + v2))
    }
}

func addEventsBootSequences(events *[]string, v1, v2 map[string]bool) {
	if len(v1) >= len(v2) {
		return
	}

    for e, _ := range v2 {
        if _, found := v1[e];!found {
            *events = append(*events, e)
        }
    }
}
`)
	fmt.Printf(`
func EndpointDataDiffEvents(ts time.Time, v1, v2 *EndpointData) ([]string, []string) {
	events := []string{}
	bootEvents := []string{}
	timestamp := strconv.FormatInt(ts.Unix() * 1000, 10)
	addEventsBootSequences(&bootEvents, v1.BootSequence.Events, v2.BootSequence.Events)
`)
	for i := 0; i < v.NumField(); i++ {
		rfield := v.Field(i)
		rfieldType := t.Field(i)
		fieldName := rfieldType.Name
		jsonName := rfieldType.Tag.Get("json")
		if jsonName == "" {
			continue
		}

		// Only manage exported fields
		if strings.ToUpper(string(fieldName[0])) != string(fieldName[0]) || fieldName == "ODAAllHostsCache" {
			continue
		}
		val := rfield.Interface()

		switch val.(type) {
		case string:
			fmt.Printf("addEventsString(&events, timestamp, \"%s\", v1.%s, v2.%s)\n", jsonName, fieldName, fieldName)
		case map[int]int:
			fmt.Printf("addEventsMapIntInt(&events, timestamp, \"%s\", v1.%s, v2.%s)\n", jsonName, fieldName, fieldName)
		case map[string]int:
			fmt.Printf("addEventsMapStringInt(&events, timestamp, \"%s\", v1.%s, v2.%s)\n", jsonName, fieldName, fieldName)
		case map[string]map[int]int:
			fmt.Printf("addEventsMapStringIntInt(&events, timestamp, \"%s\", v1.%s, v2.%s)\n", jsonName, fieldName, fieldName)
		case map[string]map[string]int:
			fmt.Printf("addEventsMapStringStringInt(&events, timestamp, \"%s\", v1.%s, v2.%s)\n", jsonName, fieldName, fieldName)

		}
	}

	fmt.Printf("\nreturn bootEvents, events\n")
	fmt.Printf("}\n")
}
