package elru

import (
	"encoding/json"
	"reflect"
	"strconv"
	"testing"

	"github.com/inverse-inc/go-utils/sharedutils"
)

func TestSimpleEncodableLRU(t *testing.T) {
	o, err := NewEncodableLRU(2)
	sharedutils.CheckError(err)

	expected := []string{}

	for i := 0; i < 5; i++ {
		stri := strconv.Itoa(i)
		if i >= 3 {
			expected = append(expected, stri)
		}

		o.Add(stri, i)
	}

	if o.Len() != len(expected) {
		t.Error("Length isn't the expected one")
	}

	for _, i := range expected {
		if v, found := o.Get(i); found {
			if i != strconv.Itoa(v.(int)) {
				t.Errorf("Unexpected value '%s'. Expected '%d'", i, v.(int))
			}
		} else {
			t.Error("Unable to find key", i)
		}
	}

	b, err := json.Marshal(o)
	sharedutils.CheckError(err)
	o2 := NewEncodableLRUForUnmarshal(reflect.TypeOf(0))
	err = json.Unmarshal(b, o2)
	sharedutils.CheckError(err)

	if o.maxSize != o2.maxSize {
		t.Error("maxSize wasn't restored properly")
	}

	if o.Len() != len(expected) {
		t.Error("Length isn't the expected one")
	}

	for _, i := range expected {
		if v, found := o2.Get(i); found {
			if i != strconv.Itoa(v.(int)) {
				t.Errorf("Unexpected value '%s'. Expected '%d'", i, v.(int))
			}
		} else {
			t.Error("Unable to find key", i)
		}
	}

}

type TestStruct struct {
	Age int `json:"age"`
}

func TestComplexEncodableLRU(t *testing.T) {
	o, err := NewEncodableLRU(2)
	sharedutils.CheckError(err)

	expected := []string{}

	for i := 0; i < 5; i++ {
		stri := strconv.Itoa(i)
		if i >= 3 {
			expected = append(expected, stri)
		}

		o.Add(stri, TestStruct{Age: i})
	}

	if o.Len() != len(expected) {
		t.Error("Length isn't the expected one")
	}

	for _, i := range expected {
		if v, found := o.Get(i); found {
			if i != strconv.Itoa(v.(TestStruct).Age) {
				t.Errorf("Unexpected value '%s'. Expected '%d'", i, v.(int))
			}
		} else {
			t.Error("Unable to find key", i)
		}
	}

	b, err := json.Marshal(o)
	sharedutils.CheckError(err)
	o2 := NewEncodableLRUForUnmarshal(reflect.TypeOf(TestStruct{}))
	err = json.Unmarshal(b, o2)
	sharedutils.CheckError(err)

	if o.maxSize != o2.maxSize {
		t.Error("maxSize wasn't restored properly")
	}

	if o.Len() != len(expected) {
		t.Error("Length isn't the expected one")
	}

	for _, i := range expected {
		if v, found := o2.Get(i); found {
			if i != strconv.Itoa(v.(TestStruct).Age) {
				t.Errorf("Unexpected value '%s'. Expected '%d'", i, v.(int))
			}
		} else {
			t.Error("Unable to find key", i)
		}
	}

}
