package fcdnshandler

import (
	"encoding/hex"
	"net"
	"testing"

	"git.inverse.ca/inverse/fingerbank-collector/common"
)

func TestHandle(t *testing.T) {
	collector := fccommon.NewFingerbankCollector("")
	h := NewDNSHandler(collector).(*DNSHandler)

	rawPacketHex := "00c09f32418c00e018b10cad08004500003c0000400040116543c0a8aa08c0a8aa14801b00350028af6175c00100000100000000000003777777066e6574627364036f72670000010001"
	rawPacket, _ := hex.DecodeString(rawPacketHex)

	l4PayloadHex := "75c00100000100000000000003777777066e6574627364036f72670000010001"
	l4Payload, _ := hex.DecodeString(l4PayloadHex)

	//Create dummy IP/MAC entry
	collector.UpdateIPMAC("2.3.4.5", "001122334455")

	h.Handle(fccommon.PacketInfo{SrcIP: "2.3.4.5"}, l4Payload, rawPacket)

	ed, err := collector.FindEndpointDataIP("2.3.4.5")

	if err != nil {
		t.Error("DNS handler didn't create an EndpointData entry")
	}

	if ed.DestinationHosts["www.netbsd.org"][0] != 1 {
		t.Error("DNS handler didn't find the destination host correctly")
	}

	h.Handle(fccommon.PacketInfo{SrcIP: "2.3.4.5"}, l4Payload, rawPacket)

	if ed.DestinationHosts["www.netbsd.org"][0] != 2 {
		t.Error("DNS handler didn't find the destination host correctly")
	}
}

func BenchmarkHandle(b *testing.B) {
	h := NewDNSHandler(fccommon.NewFingerbankCollector("")).(*DNSHandler)

	rawPacketHex := "00c09f32418c00e018b10cad08004500003c0000400040116543c0a8aa08c0a8aa14801b00350028af6175c00100000100000000000003777777066e6574627364036f72670000010001"
	rawPacket, _ := hex.DecodeString(rawPacketHex)

	l4PayloadHex := "75c00100000100000000000003777777066e6574627364036f72670000010001"
	l4Payload, _ := hex.DecodeString(l4PayloadHex)

	for i := 0; i < b.N; i++ {
		h.Handle(fccommon.PacketInfo{SrcIP: "2.3.4.5", SrcMAC: net.HardwareAddr{0xff, 0xbb, 0xee, 0x00, 0x01, 0x02}}, l4Payload, rawPacket)
	}

}
