package fcexamplehandler

import (
	"fmt"

	"git.inverse.ca/inverse/fingerbank-collector/common"
)

type ExampleHandler struct {
	fccommon.GenericHandler
}

const (
	Name       = "example"
	DefaultBPF = "(udp port 69) or (host zammitcorp.com and port 1234)"
)

// Create a new handler
func NewExampleHandler(collector fccommon.Collector) fccommon.ProtocolHandler {
	eh := &ExampleHandler{}
	eh.Collector = collector
	return eh
}

// Extract the User-Agent from the application layer
// User-Agent is simply bytes 12 to the end of the l4Payload so only need to go from bytes to string
func (h *ExampleHandler) extractUserAgent(l4Payload []byte) string {
	bytes := make([]byte, 0)
	for i := 0; i < len(l4Payload); i++ {
		if i >= 12 {
			bytes = append(bytes, l4Payload[i])
		}
	}
	return string(bytes)
}

// Extract the MAC address from the application layer
// Its the first 12 bytes of the L4 payload
func (h *ExampleHandler) extractMAC(l4Payload []byte) string {
	bytes := make([]byte, 0)
	for i := 0; i < 12; i++ {
		bytes = append(bytes, l4Payload[i])
	}
	return string(bytes)
}

// Handle function to process a packet
// info is the PacketInfo associated to this connection
// l4Payload is the bytes of the TCP or UDP packet
// rawPacket is the bytes of the packet as received on the wire
func (h *ExampleHandler) Handle(info fccommon.PacketInfo, l4Payload []byte, rawPacket []byte) {
	err := h.Collector.UpdateIPMAC(info.SrcIP, h.extractMAC(l4Payload))
	if err != nil {
		fmt.Println("Couldn't extract IP/MAC from packet")
		return
	}

	ed, _ := h.Collector.FindEndpointDataMAC(h.extractMAC(l4Payload))
	ed.UpdateMostAccurateUserAgent(h.extractUserAgent(l4Payload))

	ed.AddDestinationHost("zamm.it", 0)

	fmt.Println("Example has handled the packet")
}

// Returns the BPF of the packets the protocol handler is interested in
func (h *ExampleHandler) GetBPF() string {
	return DefaultBPF
}

// Friendly string representation of this protocol handler
func (h *ExampleHandler) String() string {
	return "Example Handler"
}

// Name that represents this protocol handler
// Should return a value following the golang package naming standards
func (h *ExampleHandler) Name() string {
	return Name
}
