package host_analyser

import (
	"context"

	"git.inverse.ca/inverse/fingerbank-api-golang/filter_engine"
)

var DefaultHostAnalyser *HostAnalyser

func init() {
	DefaultHostAnalyser = NewHostAnalyser(make(map[int][]filter_engine.Condition))
}

type HostAnalyser struct {
	strictHostWhitelist map[int][]filter_engine.Condition
}

type InvalidHostResult struct {
	Message string `json:"message"`
	Host    string `json:"host"`
	Type    string `json:"type"`
}

func NewHostAnalyser(strictHostWhitelist map[int][]filter_engine.Condition) *HostAnalyser {
	return &HostAnalyser{
		strictHostWhitelist: strictHostWhitelist,
	}
}

func (ha *HostAnalyser) AffectsDevices() []int {
	devices := []int{}
	for id, _ := range ha.strictHostWhitelist {
		devices = append(devices, id)
	}
	return devices
}

func (ha *HostAnalyser) AnalyseHosts(deviceId int, hosts map[string]map[int]int) []InvalidHostResult {
	results := make([]InvalidHostResult, 0)
	results = append(results, ha.AnalyseStrictHostWhitelist(deviceId, hosts)...)
	return results
}

func (ha *HostAnalyser) AnalyseStrictHostWhitelist(deviceId int, hosts map[string]map[int]int) []InvalidHostResult {
	ctx := context.Background()

	results := make([]InvalidHostResult, 0)
	list := ha.strictHostWhitelist[deviceId]

	if len(list) == 0 {
		return make([]InvalidHostResult, 0)
	}

	for host, _ := range hosts {
		var found bool
		for _, c := range list {
			if c.Matches(ctx, host) {
				found = true
				break
			}
		}
		if !found {
			results = append(results, InvalidHostResult{Message: "Not found in the strict host whitelist", Host: host, Type: "strict_whitelist"})
		}
	}

	return results
}
