package host_analyser

import (
	"testing"

	"git.inverse.ca/inverse/fingerbank-api-golang/filter_engine"
)

func TestAnalyseStrictHostWhitelist(t *testing.T) {
	anal := NewHostAnalyser(map[int][]filter_engine.Condition{
		1: []filter_engine.Condition{
			filter_engine.NewEqualCondition("nest.something.com"),
		},
		2: []filter_engine.Condition{
			filter_engine.NewEqualCondition("www.test.com"),
		},
	})

	var hosts map[string]map[int]int
	var results []InvalidHostResult

	hosts = map[string]map[int]int{
		"nest.something.com": map[int]int{},
		"www.test.com":       map[int]int{},
	}

	// Should only hit on www.test.com since nest.something.com is valid for this device ID
	results = anal.AnalyseStrictHostWhitelist(1, hosts)

	if len(results) != 1 {
		t.Error("Unexpected results length", len(results))
	}

	if results[0].Host != "www.test.com" {
		t.Error("Unexpected host in result")
	}

	// Should only hit on nest.something.com since www.test.com is valid for this device ID
	results = anal.AnalyseStrictHostWhitelist(2, hosts)

	if len(results) != 1 {
		t.Error("Unexpected results length", len(results))
	}

	if results[0].Host != "nest.something.com" {
		t.Error("Unexpected host in result")
	}

	// Shouldn't hit on anything since no rules are present for that device ID
	results = anal.AnalyseStrictHostWhitelist(3, hosts)

	if len(results) != 0 {
		t.Error("Unexpected results length", len(results))
	}

	// Shouldn't hit on anything since all hosts are valid
	hosts = map[string]map[int]int{
		"nest.something.com": map[int]int{},
	}

	results = anal.AnalyseStrictHostWhitelist(1, hosts)

	if len(results) != 0 {
		t.Error("Unexpected results length", len(results))
	}

	// Shouldn't hit on anything since there are no hosts
	hosts = map[string]map[int]int{}

	results = anal.AnalyseStrictHostWhitelist(1, hosts)

	if len(results) != 0 {
		t.Error("Unexpected results length", len(results))
	}

}
