package main

import (
	"flag"
	"net"

	"github.com/inverse-inc/go-utils/netflow5"
	"github.com/inverse-inc/go-utils/netflow5/processor"
	"github.com/inverse-inc/go-utils/sharedutils"
)

type netflow5Handler struct {
}

func (h *netflow5Handler) HandleFlow(header *netflow5.Header, i int, flow *netflow5.Flow) {
	//fmt.Printf("%02d) src = %s:%d, dst = %s:%d\n", i, flow.SrcIP().String(), flow.SrcPort(), flow.DstIP().String(), flow.DstPort())
	var proto string
	if flow.Proto == 6 {
		proto = "tcp"
	} else if flow.Proto == 17 {
		proto = "udp"
	} else {
		proto = "*"
	}
	collector.HandleIPPacket(proto, flow.SrcIP(), flow.DstIP(), int(flow.SrcPort()), int(flow.DstPort()))
}

var flowHandler = netflow5Handler{}

type Netflow5Service struct {
	processor processor.Processor
	disable   bool
}

var netflow5Service = Netflow5Service{
	processor: processor.Processor{
		Handler: processor.FlowToFlowsHandler(&flowHandler),
	},
	disable: false,
}

func (s *Netflow5Service) Start() {
	if !netflow5Service.disable {
		conn, err := net.ListenUDP("udp", &net.UDPAddr{
			Port: 2055,
			IP:   net.ParseIP("0.0.0.0"),
		})
		sharedutils.CheckError(err)
		netflow5Service.processor.Conn = conn
		s.processor.Start()
	}
}

func (s *Netflow5Service) Stop() {
	if !netflow5Service.disable {
		s.processor.StopAndWait()
	}
}

func init() {
	flag.BoolVar(
		&netflow5Service.disable,
		"disable-netflow5-service",
		(sharedutils.IsEnabled(sharedutils.EnvOrDefault("COLLECTOR_DISABLE_NETFLOW_SERVICE", "false")) || netflow5Service.disable),
		"Disables the netflow5 service (environment variable: COLLECTOR_DISABLE_NETFLOW_SERVICE)",
	)

}
