//go:build pfconnector

package packetfence

import (
	"context"
	"fmt"
	"os"
	"regexp"

	ps "github.com/mitchellh/go-ps"
)

func RunsOnPacketFence(ctx context.Context) bool {
	return CheckDirs(ctx) &&
		ValidatePackages(ctx) &&
		Processes(ctx)
}

func CheckDirs(ctx context.Context) bool {
	if _, err := os.Stat("/usr/local/collector-remote"); os.IsNotExist(err) {
		fmt.Println("Failed check 2425f762-44ba-11ed-b878-0242ac120002")
		return false
	}
	if _, err := os.Stat("/usr/local/pfconnector-remote"); os.IsNotExist(err) {
		fmt.Println("Failed check 5fa63e14-44ba-11ed-b878-0242ac120002")
		return false
	}
	return true
}

func ValidatePackages(ctx context.Context) bool {
	return checkPackageInstall(ctx, "packetfence-pfconnector") &&
		checkPackageInstall(ctx, "fingerbank-collector-remote")
}

func Processes(ctx context.Context) bool {
	checks := map[*regexp.Regexp]bool{
		regexp.MustCompile(`pfconnector`): false,
	}

	processes, _ := ps.Processes()

	for _, process := range processes {
		for re, alreadyMatched := range checks {
			if !alreadyMatched && re.MatchString(process.Executable()) {
				checks[re] = true
			}
		}
	}

	for _, matched := range checks {
		if !matched {
			fmt.Println("Failed check 96a82b0c-44ba-11ed-b878-0242ac120002")
			return false
		}
	}
	return true
}
